package lightcontainer.domains.client;

import lightcontainer.interfaces.ClientHandlerFFE;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.ProtocolReader;
import lightcontainer.protocol.rules.reader.SigninRule;
import lightcontainer.protocol.rules.writer.SignError;
import lightcontainer.protocol.rules.writer.SignOk;

import java.io.*;
import java.net.Socket;
import java.nio.charset.StandardCharsets;

/**
 * ClientHandler
 *
 * <!> UNICAST CLIENT <!>
 * Class communicating with the client, and
 * intercepting and sending files to the client.
 *
 * @version     1.1
 * @since       1.0
 *
 * @see         Runnable
 * @see         AutoCloseable
 * @author      Jérémi NIHART <j.nihart@student.helmo.be>
 */
public class ClientHandler implements Runnable, AutoCloseable {
    // Variables
    private ClientHandlerFFE fileFrontEnd;
    private final Socket client;
    private ProtocolRepository protocolRep;
    private boolean client_run;

    /**
     * Login of client.
     * State : NULL if not connected and Login string if connected
     */
    private String login;

    private BufferedReader reader;
    private PrintWriter writer;

    // Constructor
    public ClientHandler(Socket client, ClientHandlerFFE ffe, ProtocolRepository protocolRep) {
        this.fileFrontEnd = ffe;
        this.client = client;
        this.protocolRep = protocolRep;
        this.client_run = false;
        initClient();
    }

    /**
     * Initialise the Client's Reader and Writer.
     *
     * @since       1.0
     *
     * @see         BufferedReader
     * @see         PrintWriter
     */
    private void initClient() {
        try {
            this.reader = new BufferedReader(new InputStreamReader(
                this.client.getInputStream(),
                StandardCharsets.UTF_8
            ));
            this.writer = new PrintWriter(new OutputStreamWriter(
                this.client.getOutputStream(),
                StandardCharsets.UTF_8
            ), true);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Thread Function
     * Start the dialogue with the client.
     *
     * @since       1.0
     */
    @Override
    public void run() {
        this.client_run = true;
        while (this.client_run) {
            try {
                String command = this.reader.readLine();
                // TODO gestion de la réception de commandes, fichier, ...
                if (command != null) System.out.println("Client: " + command);

                ProtocolReader.ProtocolResult ruleResult = protocolRep.executeReader(command + "\r\n");

                if (isConnected()) {
                    ruleResult.read(
                            this.client.getInputStream()
                    );

                    if (ruleResult.getReceiver() == ProtocolReader.ResultCmdReceiver.STOREBACKEND) {
                        fileFrontEnd.newCommand(command, ruleResult.getResultCommand()); // Envoie dans la file de tâche FileFrontEnd en attente d'un traitement d'un StorBackEnd
                    } else {
                        writer.write(ruleResult.getResultCommand()); // Renvoye au client
                        writer.flush();
                    }

                } else {
                    authentication(ruleResult);
                }
            } catch (IOException ignore) { }
        }
    }

    private void authentication(ProtocolReader.ProtocolResult ruleResult) {
        try {
            SigninRule.Result signinResult = (SigninRule.Result) ruleResult;
            if (signinResult.checkCredentials()) {
                this.login = signinResult.getLogin();
                writer.write(protocolRep.executeWriter(SignOk.NAME));
                writer.flush();
                return;
            }
        } catch (ClassCastException ignored) {}

        writer.write(protocolRep.executeWriter(SignError.NAME)); // Envoie SignError car echec de la connection
        writer.flush();
        this.close(); // Fermeture de la connection
    }

    /**
     * Permet de savoir si l'utilisateur s'est connecté (Avec login et mdp)
     * @return
     */
    private boolean isConnected() {
        return login != null;
    }

    /**
     * AutoClosable Function
     * Close the Client thread and resources.
     *
     * @since       1.0
     */
    @Override
    public void close() {
        if (this.client_run) {
            try {
                this.client_run = false;
                this.client.close();
            } catch (IOException ignored) { }
        }
    }
}
